@@ -1,7 +1,7 @@
-from dexy.handler import DexyHandler
+from dexy.dexy_filter import DexyFilter
 import textile
 
-class TextileHandler(DexyHandler):
+class TextileHandler(DexyFilter):
     INPUT_EXTENSIONS = ['.txt', '.textile']
     OUTPUT_EXTENSIONS = ['.html']
     ALIASES = ['pytextile']
